@@ -1,13 +1,13 @@
 package TOML::Parser;
-use 5.010000;
+use 5.008005;
 use strict;
 use warnings;
+use utf8;
 use Encode;
 
-our $VERSION = "0.90_01";
+our $VERSION = "0.01";
 
 use TOML::Parser::Tokenizer qw/:constant/;
-use TOML::Parser::Tokenizer::Strict;
 use TOML::Parser::Util qw/unescape_str/;
 use Types::Serialiser;
 
@@ -17,7 +17,6 @@ sub new {
     return bless +{
         inflate_datetime => sub { $_[0] },
         inflate_boolean  => sub { $_[0] eq 'true' ? Types::Serialiser::true : Types::Serialiser::false },
-        strict_mode      => 0,
         %$args,
     } => $class;
 }
@@ -34,11 +33,6 @@ sub parse_fh {
     return $self->parse($src);
 }
 
-sub _tokenizer_class {
-    my $self = shift;
-    return $self->{strict_mode} ? 'TOML::Parser::Tokenizer::Strict' : 'TOML::Parser::Tokenizer';
-}
-
 our @TOKENS;
 our $ROOT;
 our $CONTEXT;
@@ -47,78 +41,65 @@ sub parse {
 
     local $ROOT    = {};
     local $CONTEXT = $ROOT;
-    local @TOKENS  = $self->_tokenizer_class->tokenize($src);
+    local @TOKENS  = TOML::Parser::Tokenizer->tokenize($src);
+    return $self->_parse_tokens();
+}
+
+sub _parse_tokens {
+    my $self = shift;
+
     while (my $token = shift @TOKENS) {
-        $self->_parse_token($token);
-    }
-    return $ROOT;
-}
-
-sub _parse_token {
-    my ($self, $token) = @_;
-
-    my ($type, $val) = @$token;
-    if ($type eq TOKEN_TABLE) {
-        $self->_parse_table($val);
-    }
-    elsif ($type eq TOKEN_ARRAY_OF_TABLE) {
-        $self->_parse_array_of_table($val);
-    }
-    elsif (my ($key, $value) = $self->_parse_key_and_value($token)) {
-        die "Duplicate key. key:$key" if exists $CONTEXT->{$key};
-        $CONTEXT->{$key} = $value;
-    }
-    elsif ($type eq TOKEN_COMMENT) {
-        # pass through
-    }
-    else {
-        die "Unknown case. type:$type";
-    }
-}
-
-sub _parse_key_and_value {
-    my ($self, $token) = @_;
-
-    my ($type, $val) = @$token;
-    if ($type eq TOKEN_KEY) {
-        my $token = shift @TOKENS;
-
-        my $key = $val;
-        my $value = $self->_parse_value_token($token);
-        return ($key, $value);
+        my ($type, $pos, $val) = @$token;
+        if ($type eq TOKEN_TABLE) {
+            $self->_parse_table($val);
+        }
+        elsif ($type eq TOKEN_ARRAY_OF_TABLE) {
+            $self->_parse_array_of_table($val);
+        }
+        elsif ($type eq TOKEN_KEY) {
+            my $token = shift @TOKENS;
+            $CONTEXT->{$val} = $self->_parse_value_token($token);
+        }
+        elsif ($type eq TOKEN_COMMENT) {
+            # pass through
+        }
+        else {
+            die "Unknown case. type:$type";
+        }
     }
 
-    return;
+    return $CONTEXT;
 }
 
 sub _parse_table {
-    my ($self, $keys) = @_;
-    my @keys = @$keys;
+    my ($self, $key) = @_;
 
-    $CONTEXT = $ROOT;
-    for my $k (@keys) {
+    local $CONTEXT = $ROOT;
+    for my $k (split /\./, $key) {
         if (exists $CONTEXT->{$k}) {
             $CONTEXT = ref $CONTEXT->{$k} eq 'ARRAY' ? $CONTEXT->{$k}->[-1] :
                        ref $CONTEXT->{$k} eq 'HASH'  ? $CONTEXT->{$k}       :
-                       die "invalid structure. @{[ join '.', @keys ]} cannot be `Table`";
+                       die "invalid structure. $key cannot be `Table`";
         }
         else {
             $CONTEXT = $CONTEXT->{$k} ||= +{};
         }
     }
+
+    $self->_parse_tokens();
 }
 
 sub _parse_array_of_table {
-    my ($self, $keys) = @_;
-    my @keys     = @$keys;
+    my ($self, $key) = @_;
+    my @keys     = split /\./, $key;
     my $last_key = pop @keys;
 
-    $CONTEXT = $ROOT;
+    local $CONTEXT = $ROOT;
     for my $k (@keys) {
         if (exists $CONTEXT->{$k}) {
             $CONTEXT = ref $CONTEXT->{$k} eq 'ARRAY' ? $CONTEXT->{$k}->[-1] :
                        ref $CONTEXT->{$k} eq 'HASH'  ? $CONTEXT->{$k}       :
-                       die "invalid structure. @{[ join '.', @keys ]} cannot be `Array of table`.";
+                       die "invalid structure. $key cannot be `Array of table`.";
         }
         else {
             $CONTEXT = $CONTEXT->{$k} ||= +{};
@@ -126,20 +107,21 @@ sub _parse_array_of_table {
     }
 
     $CONTEXT->{$last_key} = [] unless exists $CONTEXT->{$last_key};
-    die "invalid structure. @{[ join '.', @keys ]} cannot be `Array of table`" unless ref $CONTEXT->{$last_key} eq 'ARRAY';
+    die "invalid structure. $key cannot be `Array of table`" unless ref $CONTEXT->{$last_key} eq 'ARRAY';
     push @{ $CONTEXT->{$last_key} } => $CONTEXT = {};
+
+    $self->_parse_tokens();
 }
 
 sub _parse_value_token {
     my $self  = shift;
     my $token = shift;
 
-    my ($type, $val, @args) = @$token;
+    my ($type, $pos, $val) = @$token;
     if ($type eq TOKEN_COMMENT) {
         return; # pass through
     }
     elsif ($type eq TOKEN_INTEGER || $type eq TOKEN_FLOAT) {
-        $val =~ tr/_//d;
         return 0+$val;
     }
     elsif ($type eq TOKEN_BOOLEAN) {
@@ -149,48 +131,19 @@ sub _parse_value_token {
         return $self->inflate_datetime($val);
     }
     elsif ($type eq TOKEN_STRING) {
-        my ($is_raw) = @args;
-        return $is_raw ? $val : unescape_str($val);
-    }
-    elsif ($type eq TOKEN_MULTI_LINE_STRING_BEGIN) {
-        my ($is_raw) = @args;
-        my $value = $self->_parse_value_token(shift @TOKENS);
-        $value =~ s/\A(?:\r\n|[\r\n])//msg;
-        $value =~ s/\\\s+//msg;
-        if (my $token = shift @TOKENS) {
-            my ($type) = @$token;
-            return $value if $type eq TOKEN_MULTI_LINE_STRING_END;
-            die "Unexpected token: $type";
-        }
-    }
-    elsif ($type eq TOKEN_INLINE_TABLE_BEGIN) {
-        my %data;
-        while (my $token = shift @TOKENS) {
-            last if $token->[0] eq TOKEN_INLINE_TABLE_END;
-            next if $token->[0] eq TOKEN_COMMENT;
-            my ($key, $value) = $self->_parse_key_and_value($token);
-            die "Duplicate key. key:$key" if exists $data{$key};
-            $data{$key} = $value;
-        }
-        return \%data;
+        return unescape_str($val);
     }
     elsif ($type eq TOKEN_ARRAY_BEGIN) {
         my @data;
-
-        my $last_token;
         while (my $token = shift @TOKENS) {
             last if $token->[0] eq TOKEN_ARRAY_END;
-            next if $token->[0] eq TOKEN_COMMENT;
-            if ($self->{strict_mode}) {
-                die "Unexpected token: $token->[0]" if defined $last_token && $token->[0] ne $last_token->[0];
-            }
             push @data => $self->_parse_value_token($token);
-            $last_token = $token;
         }
         return \@data;
     }
-
-    die "Unexpected token: $type";
+    else {
+        die "Unknown case. type:$type";
+    }
 }
 
 sub inflate_datetime {
@@ -208,8 +161,6 @@ __END__
 
 =encoding utf-8
 
-=for stopwords versa
-
 =head1 NAME
 
 TOML::Parser - simple toml parser
@@ -226,9 +177,20 @@ TOML::Parser - simple toml parser
 TOML::Parser is a simple toml parser.
 
 This data structure complies with the tests
-provided at L<https://github.com/toml-lang/toml/tree/v0.4.0/tests>.
+provided at L<https://github.com/mojombo/toml/tree/master/tests>.
 
-The v0.4.0 specification is supported.
+=head1 WHY?
+
+In my point of view, it's very difficult to maintain C<TOML::from_toml> because -so far as I understand- there's some issues.
+
+Specifically, for example, C<TOML::from_toml> doesn't interpret correctly in some cases.
+In addition, it reports wrong line number when the error occurs.
+(This is because C<TOML::from_toml> deletes the comments and blank lines before it parses.)
+
+I conclude that C<TOML::from_toml> has an architectural feet,
+and that's why I came to an idea of re-creating another implementation in order to solve the problem.
+
+I believe that this is much easier than taking other solutions.
 
 =head1 METHODS
 
@@ -250,7 +212,6 @@ Arguments can be:
 =item * C<inflate_datetime>
 
 If use it, You can replace inflate C<datetime> process.
-The subroutine of default is C<identity>. C<e.g.) sub { $_[0] }>
 
     use TOML::Parser;
     use DateTime;
@@ -267,7 +228,6 @@ The subroutine of default is C<identity>. C<e.g.) sub { $_[0] }>
 =item * C<inflate_boolean>
 
 If use it, You can replace inflate boolean process.
-The return value of default subroutine is C<Types::Serialiser::true> or C<Types::Serialiser::false>.
 
     use TOML::Parser;
 
@@ -279,18 +239,6 @@ The return value of default subroutine is C<Types::Serialiser::true> or C<Types:
         },
     );
 
-=item * C<strict_mode>
-
-TOML::Parser is using a more flexible rule for compatibility with old TOML of default.
-If make this option true value, You can parse a toml with strict rule.
-
-    use TOML::Parser;
-
-    # create new parser
-    my $parser = TOML::Parser->new(
-        strict_mode => 1
-    );
-
 =back
 
 =item my $data = $parser->parse_file($path)
@@ -303,6 +251,17 @@ Transforms a string containing toml to a perl data structure or vice versa.
 
 =back
 
+=head1 BENCHMARK
+
+benchmark: by `author/benchmark.pl`
+
+    Benchmark: timing 10000 iterations of TOML, TOML::Parser...
+          TOML: 11 wallclock secs (11.11 usr +  0.02 sys = 11.13 CPU) @ 898.47/s (n=10000)
+    TOML::Parser:  7 wallclock secs ( 6.84 usr +  0.01 sys =  6.85 CPU) @ 1459.85/s (n=10000)
+                   Rate         TOML TOML::Parser
+    TOML          898/s           --         -38%
+    TOML::Parser 1460/s          62%           --
+
 =head1 SEE ALSO
 
 L<TOML>
@@ -318,10 +277,5 @@ it under the same terms as Perl itself.
 
 karupanerura E<lt>karupa@cpan.orgE<gt>
 
-=head1 CONTRIBUTOR
-
-Olivier Mengué E<lt>dolmen@cpan.orgE<gt>
-yowcow E<lt>yowcow@cpan.orgE<gt>
-Syohei YOSHIDA E<lt>syohex@gmail.comE<gt>
-
 =cut
+
